'From Squeak3.4 of 1 March 2003 [latest update: #5170] on 7 March 2003 at 3:49:05 am'!
"Change Set:		SendTreeExplorer-JW
Date:			7 March 2003
Author:			Jesse Welton

Like MessageTally, MessageSendTree traces the execution of a block in simulation.  There are three key differences:
	1) Rather than tallying the number of times each method is called in a given context, MessageSendTree traces and returns the exact tree of the sends made, in send order.
	2) MessageSendTree is designed to work with a SendTreeExplorer which presents the results in a hierarchical list view rather than flattened to a text pane.
	3) MessageSendTree allows all sends above a given method to be ignored, so that such a method can be conveniently traced without the bother of wading through a long send chain to get there."!

Object subclass: #MessageSendTree
	instanceVariableNames: 'context children '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-MessageSendTree'!

!MessageSendTree commentStamp: 'JW 3/7/2003 03:18' prior: 0!
Like MessageTally, MessageSendTree traces the execution of a block in simulation.  There are three key differences:
	1) Rather than tallying the number of times each method is called in a given context, MessageSendTree traces and returns the exact tree of the sends made, in send order.
	2) MessageSendTree is designed to work with SendTreeExplorer to present the results in a hierarchical list view rather than flattened to a text pane.
	3) MessageSendTree allows all sends above a given method to be ignored, so that such a method can be conveniently traced without the bother of wading through a long send chain to get there.

Compare the results of

	MessageSendTree traceBlockAndExplore: [5 factorial printString]

with the results of

	MessageSendTree traceBlockAndExplore: [5 factorial printString] fromMethod: #printString
!

AbstractHierarchicalList subclass: #SendTreeExplorer
	instanceVariableNames: 'roots '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-MessageSendTree'!

!SendTreeExplorer commentStamp: 'JW 3/7/2003 03:19' prior: 0!
This is a hierarchical list tool for displaying MessageSendTrees.
!

ListItemWrapper subclass: #SendTreeExplorerWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-MessageSendTree'!

!SendTreeExplorerWrapper commentStamp: 'JW 3/7/2003 03:10' prior: 0!
Support for the SendTreeExplorer.
!


!MessageSendTree methodsFor: 'as yet unclassified' stamp: 'JW 3/6/2003 23:46'!
addChild: subtree

	children addLast: subtree.
! !

!MessageSendTree methodsFor: 'as yet unclassified' stamp: 'JW 3/6/2003 23:46'!
children

	^children! !

!MessageSendTree methodsFor: 'as yet unclassified' stamp: 'JW 3/6/2003 23:42'!
context

	^context! !

!MessageSendTree methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 00:15'!
context: aContext

	context := aContext.
"	recClass := context receiver class.
	selector := recClass selectorAtMethod: context method setClass: [:c | methClass := c]."
	children := OrderedCollection new: 5.
! !

!MessageSendTree methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:01'!
contextString

	^context printString

! !


!MessageSendTree class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 00:11'!
traceBlock: aBlock
	"MessageSendTree traceBlock: [3.14 printString]"
	| tree prev stack |

	tree := self new context: aBlock.
	prev := aBlock.
	stack := OrderedCollection new.
	thisContext sender
		runSimulated: aBlock
		contextAtEachStep: [:current |
			current == prev ifFalse: [ "send or return"
				prev sender ifNotNil: [ "send"
					stack addLast: tree.
					tree := self new context: current.
					stack last addChild: tree.
				] ifNil: [ "return"
					tree := stack removeLast.
				].
				prev := current.
			].
		].
	^tree.! !

!MessageSendTree class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:23'!
traceBlock: aBlock fromMethod: rootSelector
	"MessageSendTree traceBlock: [3.14 printString] fromMethod: #printOn:base:"
	| roots tree prev stack |

	roots := OrderedCollection new.
	stack := OrderedCollection new.
	tree := nil. "tree is nil whenever above the rootSelector"
	prev := thisContext sender.
	thisContext sender
		runSimulated: aBlock
		contextAtEachStep: [:current |
			current == prev ifFalse: [ "send or return"
				prev sender ifNotNil: [ "send"
					(tree notNil or: [current selector == rootSelector]) ifTrue: [
						stack addLast: tree.
						tree := self new context: current.
						stack last
							ifNil: [roots addLast: tree]
							ifNotNil: [stack last addChild: tree].
					].
				] ifNil: [ "return"
					tree ifNotNil: [tree := stack removeLast].
				].
				prev := current.
			].
		].
	^roots.! !

!MessageSendTree class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:04'!
traceBlockAndExplore: aBlock
	"MessageSendTree traceBlockAndExplore: [3.14 printString]"
	
	SendTreeExplorer openExplorerOnRoot: (self traceBlock: aBlock).
! !

!MessageSendTree class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:50'!
traceBlockAndExplore: aBlock fromMethod: rootSelector
	"MessageSendTree traceBlockAndExplore: [3.14 printString] fromMethod: #printOn:base:"

	SendTreeExplorer openExplorerOnRoots:
		(self traceBlock: aBlock fromMethod: rootSelector).! !


!SendTreeExplorer methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:27'!
getList

	^roots collect: [:root | SendTreeExplorerWrapper with: root]
! !

!SendTreeExplorer methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 03:40'!
label

	^ roots isEmpty
		ifFalse: ['MessageSendTree']
		ifTrue: ['MessageSendTree',' (empty)']! !

!SendTreeExplorer methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 00:46'!
openExplorer
	| window listMorph |
	window _ (SystemWindow labelled: self label) model: self.
	window addMorph: (listMorph _ SimpleHierarchicalListMorph 
			on: self
			list: #getList
			selected: #getCurrentSelection
			changeSelected: #noteNewSelection:
			menu: #genericMenu:
			keystroke: nil)
		frame: (0@0 corner: 1@1).
	listMorph autoDeselect: false.
     window openInWorld.
! !

!SendTreeExplorer methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:26'!
root: sendTree

	roots := Array with: sendTree.
! !

!SendTreeExplorer methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:25'!
roots: sendTreeList

	roots := sendTreeList.
! !


!SendTreeExplorer class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:03'!
onRoot: sendTree

	^self new root: sendTree! !

!SendTreeExplorer class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:28'!
onRoots: sendTreeList

	^self new roots: sendTreeList! !

!SendTreeExplorer class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:03'!
openExplorerOnRoot: sendTree

	(self onRoot: sendTree) openExplorer.
! !

!SendTreeExplorer class methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 01:28'!
openExplorerOnRoots: sendTreeList

	(self onRoots: sendTreeList) openExplorer.
! !


!SendTreeExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 00:37'!
canBeDragged

	^false! !

!SendTreeExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'JW 3/7/2003 00:36'!
contents

	^item children collect: [:each | SendTreeExplorerWrapper with: each]! !

!SendTreeExplorerWrapper methodsFor: 'nil' stamp: 'JW 3/7/2003 01:01'!
asString

	^item contextString! !

SendTreeExplorerWrapper removeSelector: #hasContents!
SendTreeExplorer class removeSelector: #on:!
SendTreeExplorer class removeSelector: #openExplorerOn:!
SendTreeExplorer removeSelector: #explorerFor:!
SendTreeExplorer removeSelector: #openExplorerFor:!
